import { EnvironmentNode, KsBaseSymbol, SearchState } from '../types';
import { Position } from 'vscode-languageserver';
import { BasicTracker } from './tracker';
import { Environment } from './environment';
/**
 * The Symbol table is used to update and retrieve symbol type information
 */
export declare class SymbolTable implements GraphNode<SymbolTable> {
    /**
     * The root scope of this files symbol table
     */
    readonly rootScope: EnvironmentNode;
    /**
     * The parent symbol tables to this symbol table
     */
    readonly dependentTables: Set<SymbolTable>;
    /**
     * the child symbol tables to this symbol table
     */
    readonly dependencyTables: Set<SymbolTable>;
    /**
     * file uri for this symbol table
     */
    readonly uri: string;
    /**
     * the symbol table logger
     */
    readonly logger: ILogger;
    /**
     * construct a symbol table. typically this is done by the symbol table builder
     * @param rootScope the root scope of the symbol table
     * @param dependencyTables tables that this table is dependent on
     * @param dependentTable tables that are dependent on this table
     * @param uri the file uri for this symbol table
     * @param logger a logger for the symbol table
     */
    constructor(rootScope: EnvironmentNode, dependencyTables: Set<SymbolTable>, dependentTable: Set<SymbolTable>, uri: string, logger?: ILogger);
    /**
     * return itself for the graph interface
     */
    value(): SymbolTable;
    /**
     * get all adjacent nodes to this symbol table
     */
    adjacentNodes(): GraphNode<SymbolTable>[];
    /**
     * should be called when the associated file is closed. It will remove it's reference
     * if another symbol table doesn't reference it
     */
    closeSelf(): void;
    /**
     * This should be called when the associated symbol table is to be deleted
     */
    removeSelf(): void;
    /**
     * get every symbol in this file
     */
    allSymbols(): KsBaseSymbol[];
    /**
     * Get global symbols that match the requested symbol name
     * @param name symbol name
     * @param has function to determine if an environment has the appropriate symbol
     * @param state are we currently looking up or down the dependency tree
     */
    globalEnvironment(name: string, state: SearchState, searched: Set<SymbolTable>, has?: (env: Environment, lookup: string) => boolean): Maybe<Environment>;
    /**
     * Get all symbols locally scoped available at the provide position
     * @param pos position to check
     */
    scopedSymbols(pos: Position): KsBaseSymbol[];
    /**
     * Get all imported symbols relative to this script
     */
    importedSymbols(): KsBaseSymbol[];
    /**
     * Get a tracker for a given symbol under a filter condition and position
     * @param pos position to check
     * @param name name of the symbol
     * @param symbolFilter filter function for the symbol
     */
    scopedNamedTracker(pos: Position, name: string, symbolFilter?: (x: BasicTracker) => boolean): Maybe<BasicTracker>;
    /**
     * recursively move up the scope to get every file symbol
     * @param symbols symbols found so far
     * @param node current environment node
     */
    private fileSymbolsDepth;
    /**
     * get all symbol trackers locally in scope at a position
     * @param pos the position to retrieve symbols from
     */
    private scopedTrackers;
    /**
     * get a symbol tracker under the conditions of a provided filter
     * @param pos the position to check for symbols
     * @param trackerFilter the tracker filter
     */
    private scopedTracker;
    /**
     * recursively move up scopes to find most relevant symbols
     * @param trackers basic trackers found so far
     * @param pos position to search
     * @param nodes scope nodes to check
     */
    private scopedTrackersDepth;
    /**
     * recursively move up scope for more relevant symbols
     * @param pos position to search
     * @param nodes scope nodes to check
     * @param trackerFilter filter for acceptable symbols
     */
    private scopedTrackerDepth;
    /**
     * Search all for all imported symbol trackers that are accessible from this
     * environment
     */
    private importedTrackers;
    /**
     * Search all accessible environments from this environment
     * @param checked what environments have already been checked
     * @param state what is the current search state
     */
    private importedEnvironments;
    /**
     * Search all accessible environments from this environment
     * @param checked what environments have already been checked
     * @param state what is the current search state
     */
    private importedEnvironments_;
}
