import { ScopeKind } from '../../parser/types';
import { KsVariable } from '../../models/variable';
import { EnvironmentNode, KsSymbolKind, EnvironmentPath, UseResult } from '../types';
import { KsFunction } from '../../models/function';
import { KsLock } from '../../models/lock';
import { Range, Diagnostic } from 'vscode-languageserver';
import { BasicTracker } from './tracker';
import { SymbolTable } from './symbolTable';
import { Token } from '../../models/token';
import { KsParameter } from '../../models/parameter';
import { IType } from '../../typeChecker/types';
/**
 * The Symbol table builder is used to declare new symbols and track new symbols
 * in the kerboscript program
 */
export declare class SymbolTableBuilder {
    readonly uri: string;
    /**
     * The global environment for this file
     */
    private readonly global;
    /**
     * The root environment include the global environment and it's children
     */
    private readonly rootNode;
    /**
     * The active environment node
     */
    private activeNode;
    /**
     * The path in the environment
     */
    private path;
    /**
     * A cache from global symbols to their environment
     */
    private globalSymbolEnvironment;
    /**
     * A set of symbols tables that are dependencies of this table
     */
    dependencyTables: Set<SymbolTable>;
    /**
     * A set of symbol tables that are dependent on this table
     */
    dependentTables: Set<SymbolTable>;
    /**
     * logger class to reporting errors and information
     */
    logger: ILogger;
    /**
     * construct a symbol table builder
     * @param uri the file uri
     * @param logger a logger object
     */
    constructor(uri: string, logger?: ILogger);
    /**
     * Generate a symbol table. This method will update dependent tables
     * if and old version of the symbol table exists
     */
    build(): SymbolTable;
    /**
     * Rewind the current scope path for another pass
     */
    rewind(): void;
    /**
     * Get the current scope path TODO consider recomputing active scope
     * when set
     */
    getPath(): {
        path: EnvironmentPath;
        activeScope: EnvironmentNode;
    };
    /**
     * Set the current scope path
     * @param path current path
     * @param activeScope current scope
     */
    setPath(path: EnvironmentPath, activeScope: EnvironmentNode): void;
    /**
     * Add a dependency symbol table to this symbol table builder
     * @param symbolTable the dependency symbol table
     */
    linkDependency(symbolTable: SymbolTable): void;
    /**
     * Add a dependent symbol table to this symbol table builder
     * @param symbolTable the dependent symbol table
     */
    linkDependent(symbolTable: SymbolTable): void;
    /**
     * Push a new scope onto the scope stack
     * @param range the range of the new scope
     */
    beginScope(range: Range): void;
    /**
     * Pop the current scope off the stack
     */
    endScope(): void;
    /**
     * Indicate if the current scope if file scope
     */
    isFileScope(): boolean;
    /**
     * Indicate if the current scope if global scope
     */
    isGlobalScope(): boolean;
    /**
     * Returns the current scope depth
     */
    scopeDepth(): number;
    /**
     * Find all unused symbols in this symbol table builder
     */
    findUnused(): Diagnostic[];
    /**
     * Use a symbol in the symbol table
     * @param name token for the current symbol
     */
    useSymbol(name: Token): UseResult;
    /**
     * Set a variable symbol
     * @param token token for the variable to set
     */
    setBinding(token: Token): Maybe<Diagnostic>;
    /**
     * Add a new variable symbol to the table
     * @param scopeKind the requested scope type
     * @param token token for the requested variable
     * @param type type to declare variable as
     */
    declareVariable(scopeKind: ScopeKind, token: Token, range: Range, type?: IType): Maybe<Diagnostic>;
    /**
     * Add a new function symbol to the table
     * @param scopeKind the requested scope type
     * @param token token for the requested function
     * @param requiredParameters required parameters for the function
     * @param optionalParameters optional parameters for the function
     * @param returnValue does the function have a return type
     * @param type type to declare function as
     */
    declareFunction(scopeKind: ScopeKind, token: Token, range: Range, requiredParameters: number, optionalParameters: number, returnValue: boolean, type?: IType): Maybe<Diagnostic>;
    /**
     * Add a new lock symbol to the table
     * @param scopeKind the requested scope type
     * @param token token for the requested lock
     * @param type type to declare lock as
     */
    declareLock(scopeKind: ScopeKind, token: Token, range: Range, type?: IType): Maybe<Diagnostic>;
    /**
     * Add a new parameter symbol to the table
     * @param scopeKind the requested scope type
     * @param token token for the requested parameter
     * @param defaulted is the parameter defaulted
     */
    declareParameter(scopeKind: ScopeKind, token: Token, range: Range): Maybe<Diagnostic>;
    /**
     * check if the symbol exist and add a usage if it does
     * @param token token for the requested symbol
     * @param tracker symbol tracker
     * @param symbolType symbol type
     * @param expr expression context
     */
    checkUseSymbol(token: Token, tracker: Maybe<BasicTracker>, symbolType: KsSymbolKind): Maybe<Diagnostic>;
    /**
     * lookup a variable tracker
     * @param token token for the requested variable
     * @param scopeKind requested scope lookup
     */
    lookupVariableTracker(token: Token, scopeKind: ScopeKind): UseResult<KsVariable>;
    /**
     * lookup a function tracker
     * @param token token for the requested function
     * @param scopeKind requested scope lookup
     */
    lookupFunctionTracker(token: Token, scopeKind: ScopeKind): UseResult<KsFunction>;
    /**
     * lookup a lock tracker
     * @param token token for the requested lock
     * @param scopeKind requested scope lookup
     */
    lookupLockTracker(token: Token, scopeKind: ScopeKind): UseResult<KsLock>;
    /**
     * lookup a parameter tracker
     * @param token token for the requested parameter
     * @param scopeKind requested scope lookup
     */
    lookupParameterTracker(token: Token, scopeKind: ScopeKind): UseResult<KsParameter>;
    /**
     * lookup a parameter tracker
     * @param token token for the requested parameter
     * @param scopeKind requested scope lookup
     */
    lookupBindingTracker(token: Token, scopeKind: ScopeKind): UseResult<KsParameter | KsVariable>;
    /**
     * Generate a diagnostic when we expect to find a certain type of symbol but
     * find something else instead
     * @param token token for lookup
     * @param tracker found tracker
     * @param expected expected type
     */
    private wrongTypeError;
    /**
     * Find all unused symbol in a scope node that aren't used and
     * it's children
     * @param node scope node
     * @param errors cumulative errors
     */
    private findScopeNodeUnused;
    /**
     * Find all unused symbols in an environment that aren't used
     * @param environment environment to check
     * @param errors cumulative errors
     */
    private findScopeUnused;
    /**
     * Create a diagnostics for a symbol has been unused
     * @param tracker symbol tracker tha has been unused
     */
    private createUnusedDiagnostic;
    /**
     * Determine if a symbol tracker is in the current path
     * @param lookup token lookup string
     * @param scopeKind scope kind for the lookup local or global
     */
    private lookup;
    /**
     * Determine if a kind of symbol tracker is in the current path
     * @param lookup token lookup string
     * @param scopeKind scope kind for the lookup local or global
     * @param symbolKind symbol kind variable, function, etc.
     */
    private lookupKind;
    /**
     * Determine if a scope in the current path contains a symbol of interest
     * @param lookup token lookup string
     * @param scopeKind scope kind for the lookup local or global
     */
    private lookupEnvironment;
    /**
     * Retrieve the appropriate scope
     * @param type scope type global or local
     */
    private selectScopeNode;
    /**
     * Retrieve the active environment
     */
    private peekScope;
    /**
     * generate a resolver error when a declare symbol shadows with an existing on
     * @param name token for the requested symbol
     * @param symbol collided symbol
     */
    private shadowSymbolHint;
    /**
     * generate a resolver error when a declare symbol collides with an existing on
     * @param name token for the requested symbol
     * @param symbol collided symbol
     */
    private localConflictError;
    /**
     * Create a related information diagnostics to point to the location
     * of the original definition
     * @param symbol symbol in conflict
     */
    private symbolConflictInfo;
    /**
     * Create a conflict message for a related diagnostic
     * @param symbol symbol in conflict
     */
    private symbolConflictMessage;
    /**
     * Determine the location of the conflicted symbol. With an exception for
     * built in symbols
     * @param symbol symbol in conflict
     */
    private symbolConflictLocation;
}
