import { KsVariable } from '../models/variable';
import { KsFunction } from '../models/function';
import { KsLock } from '../models/lock';
import { KsParameter } from '../models/parameter';
import { Range, Location, Diagnostic } from 'vscode-languageserver';
import { IExpr, IStmt, ScopeKind } from '../parser/types';
import { BasicTracker } from './models/tracker';
import { Token } from '../models/token';
import { KsSuffix } from '../models/suffix';
import { Environment } from './models/environment';
import { TypeTracker } from './models/typeTracker';
import { IType } from '../typeChecker/types';
import { KsGrouping } from '../models/grouping';
/**
 * The result of a function scan
 */
export interface IFunctionScanResult {
    /**
     * The number of optionals parameters
     */
    optionalParameters: number;
    /**
     * The number of required parameters
     */
    requiredParameters: number;
    /**
     * Does the function return a value
     */
    return: boolean;
}
/**
 * This is the result of the symbol tables use symbol method
 */
export interface UseResult<T extends KsBaseSymbol = KsBaseSymbol> {
    /**
     * Was any error encounted while trying to use this symbol
     */
    error?: Diagnostic;
    /**
     * What was the tracker found for this lookup
     */
    tracker?: BasicTracker<T>;
}
export declare const enum TrackerKind {
    basic = 0,
    type = 1
}
/**
 * A union of basic and suffix trackers that a token may be attached too
 */
export declare type SymbolTracker = BasicTracker | TypeTracker;
/**
 * Interface for tracking symbols throughout a kerboscript
 */
export interface SymbolTrackerBase<T extends KsSymbol = KsSymbol> {
    /**
     * Information about the original declaration of the symbol
     */
    declared: IKsDeclared<T>;
}
/**
 * The path to an environment
 */
export interface EnvironmentPath {
    /**
     * the activated path
     */
    active: IStack<number>;
    /**
     * The back track path
     */
    backTrack: IStack<number>;
}
/**
 * A setting of a symbol
 */
export interface KsSet extends Location {
    /**
     * What is the type of the set
     */
    type: IType;
}
/**
 * The declared type of a symbol
 */
export interface IKsDeclared<TSymbol extends KsSymbol = KsSymbol, TType extends IType = IType> extends Location {
    /**
     * The underlying symbol declared
     */
    symbol: TSymbol;
    /**
     * The type of this declared symbol
     */
    type: TType;
}
/**
 * A real range for an environment
 */
export interface RealEnvironmentRange extends Range {
    /**
     * Discriminated union for range kind
     */
    kind: ScopeKind.local;
}
/**
 * A global range for an environment
 */
export interface GlobalEnvironmentRange {
    /**
     * Discriminated union for range kind
     */
    kind: ScopeKind.global;
}
/**
 * A type defining the range an environment is present in
 */
export declare type EnvironmentRange = RealEnvironmentRange | GlobalEnvironmentRange;
export declare const enum SearchState {
    dependents = 0,
    dependencies = 1
}
export interface EnvironmentNode {
    readonly parent: Maybe<EnvironmentNode>;
    readonly range: EnvironmentRange;
    readonly environment: Environment;
    readonly children: EnvironmentNode[];
}
export interface ISetResolverResult {
    readonly set: Maybe<Token>;
    readonly used: Token[];
}
export declare type KsSymbol = KsBaseSymbol | KsSuffix | KsGrouping;
export declare type KsBaseSymbol = KsVariable | KsFunction | KsLock | KsParameter;
export declare enum KsSymbolKind {
    variable = 0,
    function = 1,
    lock = 2,
    parameter = 3,
    suffix = 4,
    grouping = 5
}
/**
 * This representing a syntax nod that will be executed later
 */
export interface IDeferred {
    /**
     * Path to scope
     */
    path: EnvironmentPath;
    /**
     * Path to scope
     */
    activeScope: EnvironmentNode;
    /**
     * How many loops deep is the current location
     */
    loopDepth: number;
    /**
     * How many functions deep is the current location
     */
    functionDepth: number;
    /**
     * How many triggers deep is the current location
     */
    triggerDepth: number;
    /**
     * Node to be executed later
     */
    node: IStmt | IExpr;
}
/**
 * A more restrictive interface on an array to only allow stack methods
 */
export interface IStack<T> extends Pick<T[], 'pop' | 'push' | 'length'> {
    /**
     * array indexer
     */
    [index: number]: T;
    /**
     * array iterator symbol
     */
    [Symbol.iterator](): IterableIterator<T>;
}
