import { IExprVisitor, IStmtVisitor, ISuffixTermVisitor } from '../parser/types';
import { BasicBlock } from './models/basicBlock';
import * as SuffixTerm from '../parser/models/suffixTerm';
import * as Expr from '../parser/models/expr';
import * as Decl from '../parser/models/declare';
import * as Stmt from '../parser/models/stmt';
import { Script } from '../models/script';
import { FlowGraph } from './flowGraph';
export declare class ControlFlow implements ISuffixTermVisitor<(basicBlock: BasicBlock) => void>, IExprVisitor<(basicBlock: BasicBlock) => void>, IStmtVisitor<(basicBlock: BasicBlock) => Maybe<BasicBlock>> {
    /**
     * the logger to logging information
     */
    private readonly logger;
    /**
     * The tracer for logging the current stack tracer
     */
    private readonly tracer;
    /**
     * The script that is being type checked
     */
    private readonly script;
    /**
     * Loop context stack
     */
    private readonly loopContexts;
    /**
     * Function context stack
     */
    private readonly funcContexts;
    /**
     * Trigger context stack
     */
    private readonly triggerContexts;
    /**
     * Return context stack;
     */
    private readonly returnContext;
    /**
     * Boundaries of functions through this script
     */
    private readonly functionBoundaries;
    /**
     * Boundaries of triggers throughout this script
     */
    private readonly triggerBoundaries;
    /**
     * Map from each statement to it's basic block
     */
    /**
     * All nodes in this script
     */
    private nodes;
    constructor(script: Script, logger?: ILogger, tracer?: ITracer);
    /**
     * Generate a control flow diagram
     */
    flow(): Maybe<FlowGraph>;
    /**
     * Check the flow of this statement
     * @param stmt statement to check
     * @param prevBlock the previous basic block
     */
    private stmtFlow;
    /**
     * Check the flow of this expression
     * @param expr expression to check
     * @param block the current basic block
     */
    private exprFlow;
    /**
     * Check the flow of this suffix term
     * @param suffixTerm suffix term to check
     * @param block the current basic block
     */
    private suffixTermFlow;
    /**
     * Visit a variable declaration
     * @param decl declare variable statement
     * @param prevBlock current block
     */
    visitDeclVariable(decl: Decl.Var, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a lock declaration
     * @param decl declare lock statement
     * @param prevBlock current block
     */
    visitDeclLock(decl: Decl.Lock, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a function declaration
     * @param decl declare function statement
     * @param prevBlock current block
     */
    visitDeclFunction(decl: Decl.Func, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a parameter declaration
     * @param decl declare parameter statement
     * @param prevBlock current block
     */
    visitDeclParameter(decl: Decl.Param, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit an invalid statement
     * @param stmt the invalid statement
     * @param prevBlock current block
     */
    visitStmtInvalid(stmt: Stmt.Invalid, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a block statement
     * @param stmt the block statement
     * @param prevBlock current block
     */
    visitBlock(stmt: Stmt.Block, [prevBlock]: [BasicBlock]): Maybe<BasicBlock>;
    /**
     * Visit a expression statement
     * @param stmt the expression statement
     * @param prevBlock current block
     */
    visitExpr(stmt: Stmt.ExprStmt, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a on off statement
     * @param stmt the on off statement
     * @param prevBlock current block
     */
    visitOnOff(stmt: Stmt.OnOff, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a command statement
     * @param stmt the command statement
     * @param prevBlock current block
     */
    visitCommand(stmt: Stmt.Command, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the command expression statement
     * @param stmt the command expression statement
     * @param prevBlock current block
     */
    visitCommandExpr(stmt: Stmt.CommandExpr, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the unset statement
     * @param stmt the unset statement
     * @param prevBlock current block
     */
    visitUnset(stmt: Stmt.Unset, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the unlock statement
     * @param stmt the unlock statement
     * @param prevBlock current block
     */
    visitUnlock(stmt: Stmt.Unlock, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the set statement
     * @param stmt the set statement
     * @param prevBlock current block
     */
    visitSet(stmt: Stmt.Set, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the lazy global
     * @param stmt the lazy global
     * @param prevBlock current block
     */
    visitLazyGlobal(stmt: Stmt.LazyGlobal, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the if statement
     * @param stmt the if statement
     * @param prevBlock current block
     */
    visitIf(stmt: Stmt.If, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the else statement
     * @param stmt the else statement
     * @param prevBlock the current block
     */
    visitElse(stmt: Stmt.Else, [prevBlock]: [BasicBlock]): Maybe<BasicBlock>;
    /**
     * Visit the until statement
     * @param stmt the until statement
     * @param prevBlock the current block
     */
    visitUntil(stmt: Stmt.Until, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit the from statement
     * @param stmt the from statement
     * @param prevBlock current block
     */
    visitFrom(stmt: Stmt.From, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a when statement
     * @param stmt the when statement
     * @param prevBlock the current block
     */
    visitWhen(stmt: Stmt.When, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a return statement
     * @param stmt the return statement
     * @param prevBlock the current block
     */
    visitReturn(stmt: Stmt.Return, [prevBlock]: [BasicBlock]): Maybe<BasicBlock>;
    /**
     * Visit a break statement
     * @param stmt a break statement
     * @param prevBlock the current block
     */
    visitBreak(stmt: Stmt.Break, [prevBlock]: [BasicBlock]): Maybe<BasicBlock>;
    /**
     * Visit a switch statement
     * @param stmt the switch statement
     * @param prevBlock the current block
     */
    visitSwitch(stmt: Stmt.Switch, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a for statement
     * @param stmt the for statement
     * @param prevBlock the current block
     */
    visitFor(stmt: Stmt.For, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a on statement
     * @param stmt the on statement
     * @param prevBlock the current block
     */
    visitOn(stmt: Stmt.On, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a toggle statement
     * @param stmt the toggle statement
     * @param prevBlock the current block
     */
    visitToggle(stmt: Stmt.Toggle, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a wait statement
     * @param stmt the wait statement
     * @param prevBlock the current block
     */
    visitWait(stmt: Stmt.Wait, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a log statement
     * @param stmt the log statement
     * @param prevBlock the current block
     */
    visitLog(stmt: Stmt.Log, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a copy statement
     * @param stmt the copy statement
     * @param prevBlock the current block
     */
    visitCopy(stmt: Stmt.Copy, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a rename statement
     * @param stmt the rename statement
     * @param prevBlock the current block
     */
    visitRename(stmt: Stmt.Rename, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a delete statement
     * @param stmt the delete statement
     * @param prevBlock the current block
     */
    visitDelete(stmt: Stmt.Delete, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a run statement
     * @param stmt the run statement
     * @param prevBlock the current block
     */
    visitRun(stmt: Stmt.Run, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a runPath statement
     * @param stmt the runPath statement
     * @param prevBlock the current block
     */
    visitRunPath(stmt: Stmt.RunPath, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a runPathOnce statement
     * @param stmt the runPath statement
     * @param prevBlock the current block
     */
    visitRunPathOnce(stmt: Stmt.RunOncePath, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a compile statement
     * @param stmt the compile statement
     * @param prevBlock the current block
     */
    visitCompile(stmt: Stmt.Compile, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a list statement
     * @param stmt the list statement
     * @param prevBlock the current block
     */
    visitList(stmt: Stmt.List, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a empty statement
     * @param stmt the empty statement
     * @param prevBlock the previous block
     */
    visitEmpty(stmt: Stmt.Empty, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a print statement
     * @param stmt the print statement
     * @param prevBlock the current block
     */
    visitPrint(stmt: Stmt.Print, [prevBlock]: [BasicBlock]): BasicBlock;
    /**
     * Visit a invalid expression
     * @param expr the invalid expression
     * @param block the current subBlock
     */
    visitExprInvalid(expr: Expr.Invalid, [block]: [BasicBlock]): void;
    /**
     * Visit a ternary expression
     * @param expr the ternary expression
     * @param block the current subBlock
     */
    visitTernary(expr: Expr.Ternary, [block]: [BasicBlock]): void;
    /**
     * Visit a binary expression
     * @param expr the binary expression
     * @param block the current subBlock
     */
    visitBinary(expr: Expr.Binary, [block]: [BasicBlock]): void;
    /**
     * Visit a unary expression
     * @param expr the unary expression
     * @param block the current subBlock
     */
    visitUnary(expr: Expr.Unary, [block]: [BasicBlock]): void;
    /**
     * Visit a factor expression
     * @param expr the factor expression
     * @param block the current subBlock
     */
    visitFactor(expr: Expr.Factor, [block]: [BasicBlock]): void;
    /**
     * Visit a factor expression
     * @param expr the factor expression
     * @param block the current subBlock
     */
    visitSuffix(expr: Expr.Suffix, [block]: [BasicBlock]): void;
    /**
     * Visit a lambda expression
     * @param expr the lambda expression
     * @param block the current subBlock
     */
    visitLambda(expr: Expr.Lambda, [block]: [BasicBlock]): void;
    /**
     * Visit an invalid suffix term
     * @param _ invalid suffix term
     * @param __ current block
     */
    visitSuffixTermInvalid(_: SuffixTerm.Invalid, __: [BasicBlock]): void;
    /**
     * Visit a suffix trailer
     * @param suffixTerm suffix term trailer
     * @param block current block
     */
    visitSuffixTrailer(suffixTerm: SuffixTerm.SuffixTrailer, [block]: [BasicBlock]): void;
    /**
     * Visit a suffix term
     * @param suffixTerm suffix term
     * @param block current block
     */
    visitSuffixTerm(suffixTerm: SuffixTerm.SuffixTerm, [block]: [BasicBlock]): void;
    /**
     * Visit a call trailer
     * @param suffixTerm suffix term call
     * @param block current block
     */
    visitCall(suffixTerm: SuffixTerm.Call, [block]: [BasicBlock]): void;
    /**
     * Visit a hash index
     * @param _ suffix term hash index
     * @param __ current block
     */
    visitHashIndex(_: SuffixTerm.HashIndex, __: [BasicBlock]): void;
    /**
     * Visit a bracket index
     * @param suffixTerm suffix term bracket index
     * @param block current block
     */
    visitBracketIndex(suffixTerm: SuffixTerm.BracketIndex, [block]: [BasicBlock]): void;
    /**
     * Visit a delegate trailer
     * @param _ suffix term delegate
     * @param __ current block
     */
    visitDelegate(_: SuffixTerm.Delegate, __: [BasicBlock]): void;
    /**
     * Visit a literal
     * @param _ suffix term literal
     * @param __ current block
     */
    visitLiteral(_: SuffixTerm.Literal, __: [BasicBlock]): void;
    /**
     * Visit a identifier
     * @param _ suffix term identifier
     * @param __ current block
     */
    visitIdentifier(_: SuffixTerm.Identifier, __: [BasicBlock]): void;
    /**
     * Visit a grouping
     * @param suffixTerm suffix term grouping
     * @param block current block
     */
    visitGrouping(suffixTerm: SuffixTerm.Grouping, [block]: [BasicBlock]): void;
    /**
     * Create a new basic block and add it to the nodes
     */
    private createBlock;
    /**
     * Track a function context
     * @param entry what is the entry block
     * @param exit what is the exit block
     * @param track the function to perform the basic block manipulation
     */
    private trackFunc;
    /**
     * Track a loop context
     * @param entry what is the entry block
     * @param exit what is the exit block
     * @param track the loop to perform the basic block manipulation
     */
    private trackLoop;
    /**
     * Track a trigger context
     * @param entry what is the entry block
     * @param exit what is the exit block
     * @param track the trigger to perform the basic block manipulation
     */
    private trackTrigger;
    /**
     * Push a new function boundary onto the stack
     * @param boundary function boundary
     */
    private pushFuncContext;
    /**
     * Pop a function boundary from the stack
     */
    private popFuncContext;
    /**
     * Push a trigger boundary onto the stack
     * @param boundary trigger boundary
     */
    private pushTriggerContext;
    /**
     * Pop a trigger boundary from the stack
     */
    private popTriggerContext;
    /**
     * Push a loop boundary onto the stack
     * @param boundary loop boundary
     */
    private pushLoopContext;
    /**
     * Pop a loop boundary from the stack
     */
    private popLoopContext;
    /**
     * Peek the current return context from the stack
     */
    private peekReturnContext;
    /**
     * Peek the current break context from the stack
     */
    private peekBreakContext;
    /**
     * Check that the return context is correct
     * @param expected the expected return context
     */
    private checkReturnContext;
}
