import { Position, Location, Connection } from 'vscode-languageserver';
import { SymbolTracker, KsBaseSymbol } from './analysis/types';
import { AstContext } from './parser/scriptFind';
import { IFindResult } from './parser/types';
import { WorkspaceConfiguration } from './config/models/workspaceConfiguration';
import { ServerConfiguration } from './config/models/serverConfiguration';
export declare class KLS {
    /**
     * The logger used by this and all dependencies
     */
    private readonly logger;
    /**
     * The tracer used by this and all dependencies
     */
    private readonly tracer;
    /**
     * Connection to the client
     */
    private readonly connection;
    /**
     * Server version
     */
    private readonly version;
    /**
     * The current set of lint rules
     */
    private lintManager;
    /**
     * The current cased set of keywords for completions
     */
    private keywords;
    /**
     * Debounced function to bulk update the cached server documents
     */
    private readonly debouncedCacheDocuments;
    /**
     * A service to handle changes to the configuration
     */
    private readonly configurationService;
    /**
     * The document service to store and manage documents
     */
    private readonly documentService;
    /**
     * A service to take document info and generate all foldable regions
     */
    private readonly foldableService;
    /**
     * A service to provide analysis against scripts
     */
    private readonly analysisService;
    /**
     * A service for providing symbols when requested by the client
     */
    private readonly symbolService;
    /**
     * A service to resolve path in kos
     */
    private readonly resolverService;
    /**
     * A service for interacting with io
     */
    private readonly ioService;
    constructor(caseKind: CaseKind | undefined, logger: ILogger | undefined, tracer: ITracer | undefined, connection: Connection, version: string, defaultServerConfiguration: ServerConfiguration, defaultWorkspaceConfiguration: WorkspaceConfiguration);
    /**
     * Start the language server listening to requests from the client
     */
    listen(): void;
    /**
     * Initialize the server from the client connection
     * @param params initialization parameters
     */
    private onInitialize;
    /**
     * Post initialization register additional hooks and retrieve client
     * configurations
     * @param _ initialized parameters
     */
    private onInitialized;
    /**
     * Respond to completion requests from the client. This handler currently provides
     * both symbol completion as well as suffix completion.
     * @param completion the parameters describing the completion request
     * @param cancellation request cancellation token
     */
    private onCompletion;
    /**
     * This handler provider completion item resolution capability. This provides
     * additional information for the currently completion item selection
     * @param completionItem the item to resolve further
     */
    private onCompletionResolve;
    /**
     * This handler provides signature help suffixes and function within the document. This
     * provides extra context to the client such as the current parameter
     * @param positionParams the position of the signature request
     * @param cancellation request cancellation token
     */
    private onSignatureHelp;
    /**
     * This handler provide folding region capabilities. The client will ask for available folding
     * region in which this will respond with the ranges defined by #region and #endregion
     * @param foldingParams the document to preform folding analysis on
     */
    private onFoldingRange;
    /**
     * Respond to updates made to document by the client. This method
     * will parse and update the internal state of affects scripts
     * reporting errors to the client as they are discovered
     * @param document the updated document
     */
    private onChange;
    /**
     * Updates to the server configuration and the workspace ksconfig.json
     * @param serverConfiguration the server configuration
     * @param workspaceConfiguration the workspace configuration
     */
    private onConfigChange;
    /**
     * Cache all documents in the current workspace
     * @param rootVolumeUri uri to the root volume
     */
    private cacheDocuments;
    /**lintRules
     * send diagnostics to client
     * @param diagnostics diagnostics to organize and send
     */
    private sendDiagnostics;
    /**
     * Get document settings from the client. If the client does not support
     * have configurations then return the default configurations.
     */
    private getClientSettings;
    /**
     * Get the token at the provided position in the text document
     * @param pos position in the text document
     * @param uri uri of the text document
     */
    findToken(pos: Position, uri: string, ...contexts: AstContext[]): Promise<Maybe<IFindResult>>;
    /**
     * Get the declaration location for the token at the provided position
     * @param pos position in the document
     * @param uri uri of the document
     */
    getDeclarationLocation(pos: Position, uri: string): Promise<Maybe<Location>>;
    /**
     * Get all usage locations in all files
     * @param pos position in document
     * @param uri uri of document
     */
    getSymbolLocations(pos: Position, uri: string): Promise<Maybe<Location[]>>;
    /**
     * Get all symbols locally in scope at this particular script
     * @param pos position in document
     * @param uri document uri
     */
    getScopedSymbols(pos: Position, uri: string): Promise<KsBaseSymbol[]>;
    /**
     * Get all symbols imported in this particular script
     * @param uri document uri
     */
    getImportedSymbols(uri: string): Promise<KsBaseSymbol[]>;
    /**
     * Get all symbols in a provided file
     * @param uri document uri
     */
    getAllFileSymbols(uri: string): Promise<KsBaseSymbol[]>;
    /**
     * Get a function located at the current location
     * @param pos position in the document
     * @param uri document uri
     */
    getFunctionAtPosition(pos: Position, uri: string): Promise<Maybe<{
        tracker: SymbolTracker;
        index: number;
    }>>;
}
