import { IExprClass, IExprVisitor, IExpr, IExprClassVisitor, GrammarNode, Distribution, SyntaxKind, NodeDataBuilder, PartialNode } from '../types';
import * as SuffixTerm from './suffixTerm';
import * as Stmt from './stmt';
import { TokenType } from '../../models/tokentypes';
import { Range, Position } from 'vscode-languageserver';
import { NodeBase } from './base';
import { Token } from '../../models/token';
import { SymbolTracker } from '../../analysis/types';
/**
 * Expression base class
 */
export declare abstract class Expr extends NodeBase implements IExpr {
    /**
     * Return the tree node type of expression
     */
    get tag(): SyntaxKind.expr;
    /**
     * All expressions implement the accept method
     * Called when the node should execute the visitors methods
     * @param visitor visitor object
     */
    abstract accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
/**
 * Container for tokens constituting an invalid expression
 */
export declare class Invalid extends Expr {
    readonly pos: Position;
    readonly tokens: Token[];
    readonly partial?: PartialNode | undefined;
    /**
     * Invalid expression constructor
     * @param pos start position of the invalid expression
     * @param tokens all tokens in the invalid range
     * @param partial the partial if any found in the invalid range
     */
    constructor(pos: Position, tokens: Token[], partial?: PartialNode | undefined);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * Class holding all valid ternary expressions in KOS
 */
export declare class Ternary extends Expr {
    /**
     * Grammar for the ternary expression
     */
    static grammar: GrammarNode[];
    /**
     * The choose token
     */
    readonly choose: Token;
    /**
     * the true expression
     */
    readonly trueExpr: IExpr;
    /**
     * The if token
     */
    readonly ifToken: Token;
    /**
     * ternary condition expression
     */
    readonly condition: IExpr;
    /**
     * The else token
     */
    readonly elseToken: Token;
    /**
     * the false expression
     */
    readonly falseExpr: IExpr;
    /**
     * Constructor for all ternary expressions
     * @param builder the ternary builder
     */
    constructor(builder: NodeDataBuilder<Ternary>);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * Class holding all valid binary expressions in KOS
 */
export declare class Binary extends Expr {
    /**
     * Grammar for the binary expression
     */
    static grammar: GrammarNode[];
    /**
     * left expression of the operator
     */
    readonly left: IExpr;
    /**
     * The operator
     */
    readonly operator: Token;
    /**
     * right expression of the operator
     */
    readonly right: IExpr;
    /**
     * Constructor for all binary expressions
     * @param builder the binary expression builder
     */
    constructor(builder: NodeDataBuilder<Binary>);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * Class holding all valid unary expressions in KOS
 */
export declare class Unary extends Expr {
    readonly operator: Token;
    readonly factor: IExpr;
    /**
     * Grammar for the unary expressions
     */
    static grammar: GrammarNode[];
    /**
     * Unary expression constructor
     * @param operator unary operator
     * @param factor factor
     */
    constructor(operator: Token, factor: IExpr);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * Class holding expression with exponents
 */
export declare class Factor extends Expr {
    readonly suffix: IExpr;
    readonly power: Token;
    readonly exponent: IExpr;
    /**
     * Grammer for factor expressions
     */
    static grammar: GrammarNode[];
    /**
     * Factor constructor
     * @param suffix base expression
     * @param power exponent token
     * @param exponent exponent expression
     */
    constructor(suffix: IExpr, power: Token, exponent: IExpr);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * Class holding all anonymous functions
 */
export declare class Lambda extends Expr {
    readonly block: Stmt.Block;
    /**
     * Grammar for anonymous functions
     */
    static grammar: GrammarNode[];
    /**
     * Anonymous Function constructor
     * @param block the scope for the lambda
     */
    constructor(block: Stmt.Block);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * Class holding all kos suffixes
 */
export declare class Suffix extends Expr {
    readonly suffixTerm: SuffixTerm.SuffixTerm;
    colon?: Token<TokenType> | undefined;
    trailer?: SuffixTerm.SuffixTrailer | undefined;
    /**
     * Grammar for suffixes
     */
    static grammar: GrammarNode[];
    /**
     * Suffix constructor
     * @param suffixTerm base suffix term
     * @param colon optional suffix color
     * @param trailer optional suffix trailer
     */
    constructor(suffixTerm: SuffixTerm.SuffixTerm, colon?: Token<TokenType> | undefined, trailer?: SuffixTerm.SuffixTrailer | undefined);
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    /**
     * Get the most resolved type on this suffix
     */
    mostResolveTracker(): Maybe<SymbolTracker>;
    toLines(): string[];
    accept<T extends (...args: any) => any>(visitor: IExprVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
    static classAccept<T>(visitor: IExprClassVisitor<T>): T;
}
/**
 * All valid expressions
 */
export declare const validExpressions: Constructor<Expr>[];
export declare const validExprTypes: [IExprClass, Distribution][];
export declare const expr: import("../types").IGrammarUnion;
