import { IStmt, IExpr, IStmtVisitor, SyntaxKind, NodeDataBuilder, PartialNode } from '../types';
import * as Expr from './expr';
import { Range, Position } from 'vscode-languageserver';
import { NodeBase } from './base';
import { Token } from '../../models/token';
/**
 * Statement base class
 */
export declare abstract class Stmt extends NodeBase implements IStmt {
    /**
     * Return the tree node type of statement
     */
    get tag(): SyntaxKind.stmt;
    /**
     * All statement implement the accept method
     * Called when he node should execute the visitors methods
     * @param visitor visitor object
     */
    abstract accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
/**
 * Represents a malformed statement in kerboscript
 */
export declare class Invalid extends Stmt {
    readonly pos: Position;
    readonly tokens: Token[];
    readonly partial?: PartialNode | undefined;
    /**
     * Construct a new invalid statement
     * @param pos Provides the start position of this statement
     * @param tokens tokens involved in this invalid statement
     * @param partial any partial node that was recovered
     */
    constructor(pos: Position, tokens: Token[], partial?: PartialNode | undefined);
    /**
     * Convert this invalid statement into a set of line
     */
    toLines(): string[];
    /**
     * What is the start position of this statement
     */
    get start(): Position;
    /**
     * What is the end position of this statement
     */
    get end(): Position;
    /**
     * What are the ranges of this statement
     */
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Block extends Stmt {
    readonly open: Token;
    readonly stmts: Stmt[];
    readonly close: Token;
    constructor(builder: NodeDataBuilder<Block>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class ExprStmt extends Stmt {
    readonly suffix: Expr.Suffix;
    constructor(suffix: Expr.Suffix);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class OnOff extends Stmt {
    readonly suffix: Expr.Suffix;
    readonly onOff: Token;
    constructor(builder: NodeDataBuilder<OnOff>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Command extends Stmt {
    readonly command: Token;
    constructor(command: Token);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class CommandExpr extends Stmt {
    readonly command: Token;
    readonly expr: IExpr;
    constructor(builder: NodeDataBuilder<CommandExpr>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Unset extends Stmt {
    readonly unset: Token;
    readonly identifier: Token;
    constructor(builder: NodeDataBuilder<Unset>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Unlock extends Stmt {
    readonly unlock: Token;
    readonly identifier: Token;
    constructor(builder: NodeDataBuilder<Unlock>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Set extends Stmt {
    readonly set: Token;
    readonly suffix: Expr.Suffix;
    readonly to: Token;
    readonly value: IExpr;
    constructor(builder: NodeDataBuilder<Set>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class LazyGlobal extends Stmt {
    readonly atSign: Token;
    readonly lazyGlobal: Token;
    readonly onOff: Token;
    constructor(builder: NodeDataBuilder<LazyGlobal>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class If extends Stmt {
    readonly ifToken: Token;
    readonly condition: IExpr;
    readonly body: IStmt;
    readonly elseStmt?: Else;
    constructor(builder: NodeDataBuilder<If>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Else extends Stmt {
    readonly elseToken: Token;
    readonly body: IStmt;
    constructor(builder: NodeDataBuilder<Else>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Until extends Stmt {
    readonly until: Token;
    readonly condition: IExpr;
    readonly body: IStmt;
    constructor(builder: NodeDataBuilder<Until>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class From extends Stmt {
    readonly from: Token;
    readonly initializer: Block;
    readonly until: Token;
    readonly condition: IExpr;
    readonly step: Token;
    readonly increment: Block;
    readonly doToken: Token;
    readonly body: IStmt;
    constructor(builder: NodeDataBuilder<From>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class When extends Stmt {
    readonly when: Token;
    readonly condition: IExpr;
    readonly then: Token;
    readonly body: IStmt;
    constructor(builder: NodeDataBuilder<When>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Return extends Stmt {
    readonly returnToken: Token;
    readonly value?: IExpr;
    constructor(builder: NodeDataBuilder<Return>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Break extends Stmt {
    readonly breakToken: Token;
    constructor(breakToken: Token);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Switch extends Stmt {
    readonly switchToken: Token;
    readonly to: Token;
    readonly target: IExpr;
    constructor(builder: NodeDataBuilder<Switch>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class For extends Stmt {
    readonly forToken: Token;
    readonly element: Token;
    readonly inToken: Token;
    readonly collection: Expr.Suffix;
    readonly body: IStmt;
    constructor(builder: NodeDataBuilder<For>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class On extends Stmt {
    readonly on: Token;
    readonly suffix: Expr.Suffix;
    readonly body: IStmt;
    constructor(builder: NodeDataBuilder<On>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Toggle extends Stmt {
    readonly toggle: Token;
    readonly suffix: Expr.Suffix;
    constructor(builder: NodeDataBuilder<Toggle>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Wait extends Stmt {
    readonly wait: Token;
    readonly until?: Token;
    readonly expr: IExpr;
    constructor(builder: NodeDataBuilder<Wait>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Log extends Stmt {
    readonly log: Token;
    readonly expr: IExpr;
    readonly to: Token;
    readonly target: IExpr;
    constructor(builder: NodeDataBuilder<Log>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Copy extends Stmt {
    readonly copy: Token;
    readonly target: IExpr;
    readonly toFrom: Token;
    readonly destination: IExpr;
    constructor(builder: NodeDataBuilder<Copy>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Rename extends Stmt {
    readonly rename: Token;
    readonly fileVolume: Token;
    readonly identifier: Token;
    readonly target: IExpr;
    readonly to: Token;
    readonly alternative: IExpr;
    constructor(builder: NodeDataBuilder<Rename>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Delete extends Stmt {
    readonly deleteToken: Token;
    readonly target: IExpr;
    readonly from?: Token;
    readonly volume?: IExpr;
    constructor(builder: NodeDataBuilder<Delete>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Run extends Stmt {
    readonly run: Token;
    readonly identifier: Token;
    readonly path?: Token;
    readonly open?: Token;
    readonly args?: IExpr[];
    readonly close?: Token;
    readonly on?: Token;
    readonly expr?: IExpr;
    constructor(builder: NodeDataBuilder<Run>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class RunPath extends Stmt {
    readonly runPath: Token;
    readonly open: Token;
    readonly path: IExpr;
    readonly close: Token;
    readonly args?: IExpr[];
    constructor(builder: NodeDataBuilder<RunPath>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class RunOncePath extends Stmt {
    readonly runPath: Token;
    readonly open: Token;
    readonly path: IExpr;
    readonly close: Token;
    readonly args?: IExpr[];
    constructor(builder: NodeDataBuilder<RunOncePath>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Compile extends Stmt {
    readonly compile: Token;
    readonly target: IExpr;
    readonly to?: Token;
    readonly destination?: IExpr;
    constructor(builder: NodeDataBuilder<Compile>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class List extends Stmt {
    readonly list: Token;
    readonly collection?: Token;
    readonly inToken?: Token;
    readonly target?: Token;
    constructor(builder: NodeDataBuilder<List>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Empty extends Stmt {
    readonly empty: Token;
    constructor(empty: Token);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare class Print extends Stmt {
    readonly print: Token;
    readonly expr: IExpr;
    readonly at?: Token;
    readonly open?: Token;
    readonly x?: IExpr;
    readonly y?: IExpr;
    readonly close?: Token;
    constructor(builder: NodeDataBuilder<Print>);
    toLines(): string[];
    get start(): Position;
    get end(): Position;
    get ranges(): Range[];
    accept<T extends (...args: any) => any>(visitor: IStmtVisitor<T>, parameters: Parameters<T>): ReturnType<T>;
}
export declare const validStatements: Constructor<Stmt>[];
