/// <reference types="node" />
import { DiagnosticUri, DocumentInfo } from '../types';
import { DocumentService } from './documentService';
import { ResolverService } from './resolverService';
import { EventEmitter } from 'events';
declare type ChangeHandler = (diagnostics: DiagnosticUri[]) => void;
export declare interface AnalysisService {
    on(event: 'propagate', listener: ChangeHandler): this;
    emit(event: 'propagate', ...args: Parameters<ChangeHandler>): boolean;
}
export declare class AnalysisService extends EventEmitter {
    /**
     * A logger for message and error logger
     */
    private readonly logger;
    /**
     * A tracer to follow where exceptions occur
     */
    private readonly tracer;
    /**
     * Performance observer for tracking analysis speed
     */
    private readonly observer;
    /**
     * A service to load documents
     */
    private readonly documentService;
    /**
     * A service to resolver kos paths
     */
    private readonly resolverService;
    /**
     * A cache of lexical info for each document
     */
    private readonly lexicalCache;
    /**
     * A cache of semantic info for each document
     */
    private readonly semanticCache;
    /**
     * A cache of other diagnostics for each document
     */
    private readonly dependencyCache;
    /**
     * The set of files to eventually propagate changes to
     */
    private readonly changes;
    /**
     * debounce'd propagate changes to execute on a queued changes
     */
    private readonly debouncePropagateChange;
    /**
     * The current loaded standard library
     */
    private standardLibrary;
    /**
     * The current loaded celestial body library
     */
    private bodyLibrary;
    /**
     * The current requested completion case
     */
    private caseKind;
    /**
     * The current set of celestialBodies
     */
    private celestialBodies;
    /**
     * Construct a new analysis service
     * @param caseKind the typing case of the build in standard and body libraries
     * @param logger A logger to log performance and exception
     * @param tracer a tracer to location exceptions
     * @param documentService The document service to load new files from disk
     * @param resolverService A service to resolver kos paths
     */
    constructor(caseKind: CaseKind, logger: ILogger, tracer: ITracer, documentService: DocumentService, resolverService: ResolverService);
    /**
     * Set the case of the body library and standard library
     * @param caseKind case to set
     */
    setCase(caseKind: CaseKind): this;
    /**
     * Set the current bodies that should be completed
     * @param bodies
     */
    setBodies(bodies: string[]): this;
    /**
     * Validate a document in asynchronous stages. This produces diagnostics about known errors or
     * potential problems in the provided script
     * @param uri uri of the document
     * @param text source text of the document
     */
    analyzeDocument(uri: string, text: string): Promise<DiagnosticUri[]>;
    /**
     * Get a document info if it exists
     */
    loadInfo(uri: string): Promise<Maybe<DocumentInfo>>;
    /**
     * Get the full document in from the individual caches
     * @param uri document uri to check against
     */
    getInfo(uri: string): Maybe<DocumentInfo>;
    /**
     * Load full directory with initial set of diagnostics
     */
    loadDirectory(): Promise<DiagnosticUri[]>;
    /**
     * Main validation function for a document. Lexically and semantically understands a document.
     * Will additionally perform the same analysis on other run scripts found in this script
     * @param uri uri of the document
     * @param text source text of the document
     */
    private analyze;
    /**
     * Function to be debounce'd in order to propagate changes indirectly affected
     * files
     */
    private propagateChanges;
    /**
     * Add a file to to the set of files to eventually process
     * @param uri document uri to enqueue
     */
    private queueChanges;
    /**
     * Determine the document graph of the current root directory
     */
    private documentGraph;
    /**
     * Load the lexicon of all available documents
     */
    private loadLexicon;
    /**
     * Perform lexical analysis on the provided provided source text
     * @param uri uri to document
     * @param text source text of document
     */
    private analyzeLexicon;
    /**
     * Get lexical information about a document from a cache or file
     * @param uri uri to document
     */
    private getLexicon;
    /**
     * Perform semantic analysis on the an ast and provided dependency symbol tables
     * @param uri uri to document
     * @param script Ast of the document
     * @param tables symbol tables that are dependencies of this document
     */
    private analyzeSemantics;
    /**
     * Get semantic information about a document from a cache or file
     * @param uri uri to document
     */
    private getSemantics;
    /**
     * Load dependencies either from cache or by perform analysis on them as well
     * @param uri uri to document
     * @param script Ast of the document
     * @param includes include directives
     * @param loaded already loaded uri's
     */
    private loadDependencies;
    /**
     * Get dependencies for a document from a cache or file
     * @param uri uri to document
     */
    private getDependencies;
    /**
     * Get all valid uris from the documents run statements
     * @param uri uri of the calling document
     * @param runStmts run statements in the document
     * @param includes include directives within the file
     */
    private loadDocuments;
    /**
     * Load a single document from a given call location resolve to a provided path
     * @param location location of the include / run
     * @param path path to the document to load
     */
    private loadDocument;
    /**
     * Extract all diagnostics from a document info
     * @param documentInfo document info
     */
    private documentInfoDiagnostics;
    /**
     * Set all the individual caches for a document info
     * @param uri document uri to set
     * @param documentInfo document info to set
     */
    private setDocumentCache;
    /**
     * Is this partial document info actually full
     * @param partial potentially partial document info
     */
    private isFull;
    /**
     * Generate a loading diagnostic if file cannot be loaded
     * @param location location of the load statement run or #include
     * @param path kos path of the file
     */
    private loadError;
    /**
     * Get the symbol table corresponding to the active standard library. For now this
     * will just be the default library but this may allow us to eventually support multiple
     * version of kOS
     */
    private activeStandardLibrary;
    /**
     * Get the symbol table corresponding active set of celestial bodies for the user.
     * This allows for bodies other than that in stock ksp to be incorporated
     */
    private activeBodyLibrary;
}
export {};
