/// <reference types="node" />
import { EventEmitter } from 'events';
import { WorkspaceConfiguration } from '../config/models/workspaceConfiguration';
import { IConnection } from 'vscode-languageserver';
import { DocumentService } from './documentService';
import { ServerConfiguration } from '../config/models/serverConfiguration';
import { DiagnosticUri } from '../types';
export interface ChangeConfiguration {
    serverConfiguration: ServerConfiguration;
    workspaceConfiguration: WorkspaceConfiguration;
}
declare type ChangeHandler = (configurations: ChangeConfiguration) => void;
declare type ErrorHandler = (errors: DiagnosticUri[], uri: string) => void;
export declare interface ConfigurationService {
    on(event: 'change', listener: ChangeHandler): this;
    emit(event: 'change', ...args: Parameters<ChangeHandler>): boolean;
    on(event: 'error', listener: ErrorHandler): this;
    emit(event: 'error', ...args: Parameters<ErrorHandler>): boolean;
}
declare type ConfigurationConnection = Pick<IConnection, 'onDidChangeConfiguration'>;
/**
 * A service responsible for collection and organizing user
 * configurations
 */
export declare class ConfigurationService extends EventEmitter {
    /**
     * What is the current workspace's configuration
     */
    workspaceConfiguration: WorkspaceConfiguration;
    /**
     * What is the server configuration
     */
    serverConfiguration: ServerConfiguration;
    /**
     * What is the default workspace configuration
     */
    private defaultWorkspaceConfiguration;
    /**
     * Create a new configuration service
     * @param defaultServerConfiguration the default server configuration
     * @param defaultWorkSpaceConfiguration the default workspace configuration
     * @param conn connection to the client to listen to events
     * @param documentService a document server to listen to config change events
     */
    constructor(defaultServerConfiguration: ServerConfiguration, defaultWorkSpaceConfiguration: WorkspaceConfiguration, conn: ConfigurationConnection, documentService: DocumentService);
    /**
     * Update the server configuration with a diff
     * @param config config to use to update configuration with
     */
    updateServerConfiguration(config: Partial<ServerConfiguration>): void;
    /**
     * Add event listens for server and workspace config changes
     * @param conn client connection to listen for config changes
     * @param documentService document server to listen to workspace config changes
     */
    private listen;
    /**
     * Server configuration change handler. Updates the configuration when appropriate
     * @param change changes that occurred to the server configuration
     */
    private onChangeServerConfig;
    /**
     * Workspace configuration change handler. Update the configuration when appropriate
     * @param document workspace config document
     */
    private onChangeWorkspaceConfig;
}
export {};
