/// <reference types="node" />
import { IConnection } from 'vscode-languageserver';
import { TextDocument } from 'vscode-languageserver-textdocument';
import { URI } from 'vscode-uri';
import { IoService } from './ioService';
import { EventEmitter } from 'events';
declare type DocumentChangeHandler = (document: TextDocument) => void;
declare type DocumentClosedHandler = (uri: string) => void;
declare type DocumentConnection = Pick<IConnection, 'onDidChangeTextDocument' | 'onDidCloseTextDocument' | 'onDidOpenTextDocument'>;
export declare interface DocumentService {
    on(event: 'change', listener: DocumentChangeHandler): this;
    emit(event: 'change', ...args: Parameters<DocumentChangeHandler>): boolean;
    on(event: 'configChange', listener: DocumentChangeHandler): this;
    emit(event: 'configChange', ...args: Parameters<DocumentChangeHandler>): boolean;
    on(event: 'close', listener: DocumentClosedHandler): this;
    emit(event: 'close', ...args: Parameters<DocumentClosedHandler>): boolean;
}
/**
 * Service responsible for managing documents being loaded by the client and the server
 */
export declare class DocumentService extends EventEmitter {
    /**
     * What is the root volume of this directory
     */
    workspaceUri?: URI;
    /**
     * Currently cached documents loaded by the client
     */
    private clientDocs;
    /**
     * Current cached document only loaded by the server
     */
    private serverDocs;
    /**
     * A ksconfig.json if it exists reported by the client
     */
    private clientConfigDoc?;
    /**
     * A ksconfig.json if it exists read by the server
     */
    private serverConfigDoc?;
    /**
     * client connection with events for open, close, and change events
     */
    private conn;
    /**
     * A logger to log information to
     */
    private logger;
    /**
     * A tracer to location exception
     */
    private tracer;
    /**
     * A service for interacting with io
     */
    private ioService;
    /**
     * Create a new instance of the document service.
     * @param conn document connection holding the required callbacks from iconnection
     * @param ioService service to load from a provided uri
     * @param resolverService service to resolve uri for kerboscript
     * @param logger logger to log messages to client
     * @param tracer tracer to location exception
     */
    constructor(conn: DocumentConnection, ioService: IoService, logger: ILogger, tracer: ITracer, workspaceUri?: URI);
    /**
     * Attach to listener to connection events
     */
    listen(): void;
    /**
     * Is the document service read
     */
    ready(): boolean;
    /**
     * Get a document from this document service
     * @param uri uri to lookup document
     */
    getDocument(uri: string): Maybe<TextDocument>;
    /**
     * Get all document held in this service
     */
    getAllDocuments(): TextDocument[];
    /**
     * Attempt to load a document from a provided uri
     * @param uri the requested uri
     */
    loadDocument(uri: string): Promise<Maybe<TextDocument>>;
    /**
     * Cache all documents in the workspace.
     */
    cacheDocuments(): Promise<void>;
    /**
     * Retrieve a resource from the provided uri
     * @param uri uri to load resources from
     */
    private retrieveResource;
    /**
     * Handle calls to open text document.
     * @param handler handler to call when did open text document events fires
     */
    private onOpenHandler;
    /**
     * Set the client cache emit a file change event
     * @param document document to cache
     */
    private onOpenFile;
    /**
     * Set the config cache and emit a config changed events
     * @param config config to cache
     */
    private onOpenConfig;
    /**
     * Handle calls to on change text document
     * @param handler handler to call when did change text document event fires
     */
    private onChangeHandler;
    /**
     * Apply all text sequentially
     * @param document original document
     * @param changes changes to apply
     */
    private applyChanges;
    /**
     * Get the cached file for the requested uri
     * @param uri file uri
     */
    private getCache;
    /**
     * Set cached with an new text document
     * @param uri file uri
     * @param document new text document
     */
    private setCache;
    /**
     * On document close handler. Move document from editor docs to server docs.
     * This call notifies any provided handler
     * @param handler document closed handler
     */
    private onCloseHandler;
}
export {};
