/// <reference types="node" />
import { EventEmitter } from 'events';
import { DocumentService } from './documentService';
import { AnalysisService } from './analysisService';
import { DocumentSymbolParams, CancellationToken, SymbolInformation, WorkspaceSymbolParams, TextDocumentPositionParams, Location, Position, ReferenceParams, RenameParams, WorkspaceEdit, IConnection, Hover, DocumentHighlight } from 'vscode-languageserver';
import { AstContext } from '../parser/scriptFind';
import { ResolverService } from './resolverService';
import { IoService } from './ioService';
import { IFindResult } from '../parser/types';
export declare type SymbolConnection = Pick<IConnection, 'onRenameRequest' | 'onReferences' | 'onDocumentSymbol' | 'onWorkspaceSymbol' | 'onDefinition' | 'onHover' | 'onDocumentHighlight'>;
export declare class SymbolService extends EventEmitter {
    /**
     * client connection with events for open, close, and change events
     */
    private conn;
    /**
     * A service to resolve path in kos
     */
    private readonly resolverService;
    /**
     * A service for interacting with io
     */
    private readonly ioService;
    /**
     * The document service to store and manage documents
     */
    private readonly documentService;
    /**
     * A service to provide analysis against scripts
     */
    private readonly analysisService;
    /**
     * Construct a new completion service
     * @param conn Symbol connection holding hte required callbacks from iconnection
     * @param resolverService A service for resolving paths
     * @param ioService A service for interacting with io
     * @param documentService The document service to load new files from disk
     * @param analysisService The analysis service for retrieving semantic document info
     */
    constructor(conn: SymbolConnection, resolverService: ResolverService, ioService: IoService, documentService: DocumentService, analysisService: AnalysisService);
    /**
     * Attach to listener to connection events
     */
    listen(): void;
    /**
     * This handler provides document symbol capabilities. This provides a list of all
     * symbols that are located within a given document
     * @param documentSymbol the document to provide symbols for
     * @param cancellation request cancellation token
     */
    onDocumentSymbol(documentSymbol: DocumentSymbolParams, cancellation: CancellationToken): Promise<Maybe<SymbolInformation[]>>;
    /**
     * This handler provides workspace symbol capabilities
     * @param workspaceSymbol workspace symbol params
     * @param cancellation cancellation token
     */
    onWorkspaceSymbol(workspaceSymbol: WorkspaceSymbolParams, cancellation: CancellationToken): Maybe<SymbolInformation[]>;
    /**
     * This handler provides go to definition capabilities. When a client requests a symbol
     * go to definition this provides the location if it exists
     * @param positionParams the position of the definition request
     * @param cancellation request cancellation token
     */
    onDefinition(positionParams: TextDocumentPositionParams, cancellation: CancellationToken): Promise<Maybe<Location>>;
    /**
     * This handler provides reference capabilities to symbols in a document. This allows a client
     * to identify all positions that a symbol is used in the document or attached documents
     * @param reference parameters describing the reference request
     * @param cancellation request cancellation token
     */
    onReferences(reference: ReferenceParams, cancellation: CancellationToken): Promise<Maybe<Location[]>>;
    /**
     * This handler provider rename capabilities. This allows a client to highlight
     * as symbol and provide a new name that will change for all known symbols
     * @param rename information describing what and where a rename should occur
     * @param cancellation request cancellation token
     */
    onRenameRequest(rename: RenameParams, cancellation: CancellationToken): Promise<Maybe<WorkspaceEdit>>;
    /**
     * This handler provides on hover capability for symbols in a document. This allows additional
     * information to be displayed to the user about symbols throughout the document
     * @param positionParams the position of the hover request
     * @param cancellation request cancellation token
     */
    onHover(positionParams: TextDocumentPositionParams, cancellation: CancellationToken): Promise<Maybe<Hover>>;
    /**
     * This handler provides highlight within a requested document. This allows the client
     * to highlight and symbol and other instances of that symbol to also highlight.
     * @param positionParams the position of the highlight request
     * @param cancellation request cancellation token
     */
    onDocumentHighlight(positionParams: TextDocumentPositionParams, cancellation: CancellationToken): Promise<Maybe<DocumentHighlight[]>>;
    /**
     * Take a location and turn it into a document highlight
     * @param location location of symbol
     * @param kind highlight kind
     */
    private toHighlight;
    /**
     * Get all usage locations in all files
     * @param pos position in document
     * @param uri uri of document
     * @param includeDeclaration Should this include the declaration
     */
    private getSymbolLocations;
    /**
     * Get the declaration location for the token at the provided position
     * @param pos position in the document
     * @param uri uri of the document
     */
    private getDeclarationLocation;
    /**
     * Get the location of a run statement
     * @param run run statement
     * @param uri run statment uri
     */
    private getRunLocation;
    /**
     * Get the token at the provided position in the text document
     * @param pos position in the text document
     * @param uri uri of the text document
     */
    findToken(pos: Position, uri: string, ...contexts: AstContext[]): Promise<Maybe<IFindResult>>;
    /**
     * Get or load a symbol table if it exists
     * @param uri document uri to load
     */
    private loadSymbolTable;
    /**
     * Get a symbol table if it has already been loaded
     * @param uri document uri to get
     */
    private getSymbolTable;
    /**
     * load all symbols in a provided file
     * @param uri document uri
     */
    private loadAllTableSymbols;
}
