import { IType, Access, TypeKind, IParametricType, OperatorKind, IIndexer, ICallSignature } from '../../types';
import { TypeTracker } from '../../../analysis/models/typeTracker';
import { Operator } from './operator';
import { KsSymbol } from '../../../analysis/types';
/**
 * A class representing a type in Kerboscript
 */
export declare class DelegateType implements IType {
    /**
     * What is the name of this union
     */
    readonly name: string;
    /**
     * What access level does this delegate have
     */
    readonly access: Access;
    /**
     * Is this delegate the any type
     */
    readonly anyType: boolean;
    /**
     * Is this delegate the none type
     */
    readonly noneType: boolean;
    /**
     * What is the type of this delegate
     */
    readonly kind: TypeKind;
    /**
     * The tracker for this type. Should only occur if this is a suffix
     */
    private readonly typeTracker;
    /**
     * The underlying function type to the delegate
     */
    private readonly function;
    /**
     * Construct a new delegate type
     * @param type function type of this delegate
     */
    constructor(type: IType);
    /**
     * Is this delegate type a subtype of some other type
     * @param type the type to check against
     */
    isSubtypeOf(type: IParametricType): boolean;
    /**
     * Individual type check for sub type ness
     * @param type the type to check against
     */
    private subtypeCheck;
    /**
     * What is the assignment type of this type
     */
    assignmentType(): IType;
    /**
     * Get the super type
     */
    super(): Maybe<IType>;
    /**
     * Sub types of this type
     */
    subTypes(): IType[];
    /**
     * Get the type tracker
     */
    tracker(): TypeTracker<KsSymbol>;
    /**
     * Get call signature. TODO could attempt to see if the call signatures could be merged
     * somehow
     */
    callSignature(): Maybe<ICallSignature>;
    /**
     * Get indexer of this delegate type. If all types have an indexer with the same index type
     */
    indexer(): Maybe<IIndexer>;
    /**
     * Get all suffixes present in all members of the delegate type
     */
    suffixes(): Map<string, IType>;
    /**
     * Does this type have the requested suffix
     * @param name name of the suffix
     */
    hasSuffix(name: string): boolean;
    /**
     * Attempt to access a suffix from this type
     * @param name name of the suffix
     */
    getSuffix(name: string): Maybe<IType>;
    /**
     * Get an operator for the other type
     * @param kind The operator kind
     * @param rhs the type of the other type
     */
    getOperator(kind: OperatorKind, rhs?: IType): Maybe<Operator<IType>>;
    /**
     * Get all operators. TODO attempt to merge operators
     */
    operators(): Map<OperatorKind, Operator<IType>[]>;
    /**
     * Can only coerce from itself
     * @param type type to try to coerce from
     */
    canCoerceFrom(type: IParametricType): boolean;
    /**
     * Get all available coercions
     */
    coercions(): Set<IParametricType>;
    /**
     * Apply type arguments to this delegate type
     * @param _ type arguments
     */
    apply(_: IType | Map<IParametricType, IType>): IType;
    /**
     * Get a string representation of this type
     */
    toString(): string;
    /**
     * Get the type parameters of this type
     */
    getTypeParameters(): IParametricType[];
}
