import { IType, Access, ICallSignature, TypeKind, IParametricType, OperatorKind, TypeMap, IIndexer } from '../../types';
import { TypeTracker } from '../../../analysis/models/typeTracker';
import { Operator } from './operator';
/**
 * A class representing a type in Kerboscript
 */
export declare class Type implements IType {
    /**
     * Name of the type
     */
    readonly name: string;
    /**
     * What is the access level of this type
     */
    readonly access: Access;
    /**
     * What is the call signature if there is one
     */
    readonly typeCallSignature?: ICallSignature;
    /**
     * What is the type kind
     */
    readonly kind: TypeKind;
    /**
     * Is this type the any type
     */
    readonly anyType: boolean;
    /**
     * Is this type the none type
     */
    readonly noneType: boolean;
    /**
     * The type arguments for this type if any
     */
    private readonly typeArguments;
    /**
     * The tracker for this type. Should only occur if this is a suffix
     */
    private readonly typeTracker;
    /**
     * The super type if any for this type
     */
    private superType?;
    /**
     * The indexer of this type if any
     */
    private indexerSignature?;
    /**
     * The parametric type this type derives from
     */
    private readonly typeTemplate?;
    /**
     * The types that can be coerced into this type
     */
    private readonly coercibleTypes;
    /**
     * The suffixes on this type
     */
    private readonly suffixMap;
    /**
     * The operators on this type
     */
    private readonly operatorMap;
    /**
     * Construct a new kerboscript type
     * @param name name of the type
     * @param access what access does this type have
     * @param typeArguments what are the type arguments of this type
     * @param kind what is the kind of this type
     * @param callSignature what is the call signature of this type
     * @param typeTemplate what is the type template of this type
     * @param anyType is this the any type
     * @param noneType is this the none type
     */
    constructor(name: string, access: Access, typeArguments: Map<IParametricType, IType>, kind: TypeKind, callSignature?: ICallSignature, typeTemplate?: IParametricType, anyType?: boolean, noneType?: boolean);
    /**
     * Add a super type to this type
     * @param typeMap type map
     */
    addSuper(typeMap: TypeMap<IType>): void;
    /**
     * Add an indexer to this type
     * @param indexer indexer to add
     */
    addIndexer(indexer: TypeMap<IIndexer>): void;
    /**
     * Add new coercions to this type
     * @param types type to new coercions
     */
    addCoercion(...types: IType[]): void;
    /**
     * Add new suffix to this type
     * @param suffixes suffixes to add
     */
    addSuffixes(...suffixes: TypeMap<IType>[]): void;
    /**
     * Add new operators to this type
     * @param operators operators to add
     */
    addOperators(...operators: Operator<IType>[]): void;
    /**
     * Is this type a subtype for another type
     * @param type the type check against
     */
    isSubtypeOf(type: IParametricType): boolean;
    /**
     * Individual type check for sub type ness
     * @param type the type to check against
     */
    private subtypeCheck;
    /**
     * Can this type be coerced from
     * @param type
     */
    canCoerceFrom(type: IParametricType): boolean;
    /**
     * What are the type parameters of this type
     */
    getTypeParameters(): IParametricType[];
    /**
     * Get the super type of this type
     */
    super(): Maybe<IType>;
    /**
     * Sub types of this type
     */
    subTypes(): IType[];
    /**
     * Get the call signature of this type
     */
    callSignature(): Maybe<ICallSignature>;
    /**
     * Get the indexer of this type
     */
    indexer(): Maybe<IIndexer>;
    /**
     * Get the tracker of this type
     */
    tracker(): TypeTracker;
    /**
     * Get the assignment type of this type
     */
    assignmentType(): IType;
    /**
     * Get all available coercions
     */
    coercions(): Set<IParametricType>;
    /**
     * Get all available suffixes
     */
    suffixes(): Map<string, IType>;
    /**
     * Does this type has the requested suffix
     * @param name name of suffix
     */
    hasSuffix(name: string): boolean;
    /**
     * Attempt to retrieve a suffix from this type
     * @param name name of the suffix
     */
    getSuffix(name: string): Maybe<IType>;
    /**
     * Get the appropriate operator for this kind if it exists
     * @param kind the operator kind
     * @param other the other type if binary
     */
    getOperator(kind: OperatorKind, other?: Maybe<IType>): Maybe<Operator<IType>>;
    /**
     * Get all operators of this type
     */
    operators(): Map<OperatorKind, Operator<IType>[]>;
    /**
     * Get a string representation of this type
     */
    toString(): string;
    /**
     * Apply type arguments to this type
     * @param _ type arguments
     */
    apply(_: Map<IType, IType> | IType): IType;
}
