import { IType, Access, TypeKind, IParametricType, OperatorKind, IIndexer, ICallSignature } from '../../types';
import { TypeTracker } from '../../../analysis/models/typeTracker';
import { Operator } from './operator';
import { KsSymbol } from '../../../analysis/types';
/**
 * A class representing a type in Kerboscript
 */
export declare class UnionType implements IType {
    /**
     * What is the name of this union
     */
    readonly name: string;
    /**
     * What access level does this union have
     */
    readonly access: Access;
    /**
     * Is this union the any type
     */
    readonly anyType: boolean;
    /**
     * Is this union the none type
     */
    readonly noneType: boolean;
    /**
     * What is the type of this union
     */
    readonly kind: TypeKind;
    /**
     * The tracker for this type. Should only occur if this is a suffix
     */
    private readonly typeTracker;
    /**
     * What are the internal type of this union
     */
    readonly types: IType[];
    /**
     * Is this type parameter for a boolean
     */
    private readonly param;
    /**
     * Construct a new union type
     * @param isParameter Is this for a call parameter
     * @param types types of the union
     */
    constructor(isParameter: boolean, ...types: IType[]);
    /**
     * Is this union type a subtype of some other type
     * @param type the type to check against
     */
    isSubtypeOf(type: IParametricType): boolean;
    /**
     * What is the assignment type of this type
     */
    assignmentType(): IType;
    /**
     * Get the super type
     */
    super(): Maybe<IType>;
    /**
     * The sub types of this type
     */
    subTypes(): IType[];
    /**
     * Get the type tracker
     */
    tracker(): TypeTracker<KsSymbol>;
    /**
     * Get call signature. TODO could attempt to see if the call signatures could be merged
     * somehow
     */
    callSignature(): Maybe<ICallSignature>;
    /**
     * Get indexer of this union type. If all types have an indexer with the same index type
     */
    indexer(): Maybe<IIndexer>;
    /**
     * Get all suffixes present in all members of the union type
     */
    suffixes(): Map<string, IType>;
    /**
     * Does this type have the requested suffix TODO this may be an issue where
     * a union has two suffixes of the same name but not the same type
     * @param name name of the suffix
     */
    hasSuffix(name: string): boolean;
    /**
     * Attempt to retrieve a suffix from this type TODO this may be an issue where
     * a union has two suffixes of the same name but not the same type
     * @param name name of the suffix
     */
    getSuffix(name: string): Maybe<IType>;
    /**
     * Get an operator for the other type
     * @param _ The operator kind
     * @param __ the type of the other type
     */
    getOperator(_: OperatorKind, __?: IType): Maybe<Operator<IType>>;
    /**
     * Get all operators. TODO attempt to merge operators
     */
    operators(): Map<OperatorKind, Operator<IType>[]>;
    canCoerceFrom(type: IParametricType): boolean;
    /**
     * Get all available coercions
     */
    coercions(): Set<IParametricType>;
    /**
     * Apply type arguments to this union type
     * @param _ type arguments
     */
    apply(_: IType | Map<IParametricType, IType>): IType;
    /**
     * Get a string representation of this type
     */
    toString(): string;
    /**
     * Get the type parameters of this type
     */
    getTypeParameters(): IParametricType[];
}
