import { IExprVisitor, IStmtVisitor, ISuffixTermVisitor } from '../parser/types';
import * as SuffixTerm from '../parser/models/suffixTerm';
import * as Expr from '../parser/models/expr';
import * as Stmt from '../parser/models/stmt';
import * as Decl from '../parser/models/declare';
import { ITypeResultExpr, IType } from './types';
import { Script } from '../models/script';
import { Diagnostic } from 'vscode-languageserver';
import { SuffixTypeBuilder } from './models/suffixTypeNode';
declare type Diagnostics = Diagnostic[];
/**
 * The type checker attempts to identify places where the rules of the type
 * system are broken.
 */
export declare class TypeChecker implements IStmtVisitor<() => Diagnostics>, IExprVisitor<() => ITypeResultExpr<IType>>, ISuffixTermVisitor<(builder: SuffixTypeBuilder) => Diagnostics> {
    /**
     * the logger to logging information
     */
    private readonly logger;
    /**
     * The tracer for logging the current stack tracer
     */
    private readonly tracer;
    /**
     * The script that is being type checked
     */
    private readonly script;
    /**
     * The cached bound check statement
     */
    private readonly checkStmtBind;
    constructor(script: Script, logger?: ILogger, tracer?: ITracer);
    /**
     * Check the source file for type errors
     */
    check(): Diagnostics;
    /**
     * check a collection of statements
     * @param stmts statements to check
     */
    private checkStmts;
    /**
     * Check an statement for errors
     * @param stmt statement to check
     */
    private checkStmt;
    /**
     * Check an expression for errors
     * @param expr expression to check
     */
    private checkExpr;
    /**
     * Check a suffix term for errors
     * @param suffixTerm suffix term to check
     * @param builder type resolved so far
     */
    private checkSuffixTerm;
    /**
     * Visit a variable declaration
     * @param decl variable declaration
     */
    visitDeclVariable(decl: Decl.Var): Diagnostics;
    /**
     * Vist a lock declaration
     * @param decl lock declaration
     */
    visitDeclLock(decl: Decl.Lock): Diagnostics;
    /**
     * Visit a function declaration
     * @param decl function declaration
     */
    visitDeclFunction(decl: Decl.Func): Diagnostics;
    /**
     * Visit a parameter declaration
     * @param decl parameter declaration
     */
    visitDeclParameter(decl: Decl.Param): Diagnostics;
    /**
     * Visit an invalid statement
     * @param stmt invalid statement
     */
    visitStmtInvalid(stmt: Stmt.Invalid): Diagnostics;
    /**
     * Visit a block statement
     * @param stmt statement block
     */
    visitBlock(stmt: Stmt.Block): Diagnostics;
    /**
     * Visit an statement expression
     * @param stmt statement expression
     */
    visitExpr(stmt: Stmt.ExprStmt): Diagnostics;
    /**
     * Visit an on off statement
     * @param stmt on / off statement
     */
    visitOnOff(stmt: Stmt.OnOff): Diagnostics;
    /**
     * Visit a command statement
     * @param _ command statement
     */
    visitCommand(_: Stmt.Command): Diagnostics;
    /**
     * Visit a command expression statement
     * @param stmt command expression statement
     */
    visitCommandExpr(stmt: Stmt.CommandExpr): Diagnostics;
    /**
     * Visit an unset statement
     * @param _ unset statement
     */
    visitUnset(_: Stmt.Unset): Diagnostics;
    /**
     * Visit an unlock statement
     * @param _ unlock statement
     */
    visitUnlock(_: Stmt.Unlock): Diagnostics;
    /**
     * Visit set statement
     * @param stmt set statement
     */
    visitSet(stmt: Stmt.Set): Diagnostics;
    visitLazyGlobal(_: Stmt.LazyGlobal): Diagnostics;
    visitIf(stmt: Stmt.If): Diagnostics;
    visitElse(stmt: Stmt.Else): Diagnostics;
    visitUntil(stmt: Stmt.Until): Diagnostics;
    visitFrom(stmt: Stmt.From): Diagnostics;
    visitWhen(stmt: Stmt.When): Diagnostics;
    visitReturn(stmt: Stmt.Return): Diagnostics;
    visitBreak(_: Stmt.Break): Diagnostics;
    visitSwitch(stmt: Stmt.Switch): Diagnostics;
    /**
     * Visit a for loop
     * @param stmt for loop statement
     */
    visitFor(stmt: Stmt.For): Diagnostics;
    /**
     * Visit on statement
     * @param stmt on statement
     */
    visitOn(stmt: Stmt.On): Diagnostics;
    /**
     * Visit toggle statement
     * @param stmt toggle statement
     */
    visitToggle(stmt: Stmt.Toggle): Diagnostics;
    /**
     * Visit wait statement
     * @param stmt wait statement
     */
    visitWait(stmt: Stmt.Wait): Diagnostics;
    /**
     * Visit log statement
     * @param stmt log statement
     */
    visitLog(stmt: Stmt.Log): Diagnostics;
    visitCopy(stmt: Stmt.Copy): Diagnostics;
    visitRename(stmt: Stmt.Rename): Diagnostics;
    visitDelete(stmt: Stmt.Delete): Diagnostics;
    visitRun(_: Stmt.Run): Diagnostics;
    visitRunPath(stmt: Stmt.RunPath): Diagnostics;
    visitRunPathOnce(stmt: Stmt.RunOncePath): Diagnostics;
    visitCompile(stmt: Stmt.Compile): Diagnostics;
    /**
     * Visit list statement
     * @param stmt list statement
     */
    visitList(stmt: Stmt.List): Diagnostics;
    visitEmpty(_: Stmt.Empty): Diagnostics;
    visitPrint(stmt: Stmt.Print): Diagnostics;
    visitExprInvalid(_: Expr.Invalid): ITypeResultExpr<IType>;
    /**
     * Check the ternary expression TODO need union type
     * @param expr ternary expression
     */
    visitTernary(expr: Expr.Ternary): ITypeResultExpr<IType>;
    /**
     * Visit binary expression of form `a op b`
     * @param expr binary expression
     */
    visitBinary(expr: Expr.Binary): ITypeResultExpr<IType>;
    /**
     * Visit unary expression of form `op a`
     * @param expr unary expression
     */
    visitUnary(expr: Expr.Unary): ITypeResultExpr<IType>;
    /**
     * Visit a factor expression of form `a ^ b`.
     * @param expr factor expression
     */
    visitFactor(expr: Expr.Factor): ITypeResultExpr<IType>;
    /**
     * Visit a suffix
     * @param expr suffix expression
     */
    visitSuffix(expr: Expr.Suffix): ITypeResultExpr<IType>;
    /**
     * Visit a lambda expression. TODO need to generate an appropriate type for the
     * function body
     * @param expr the lambda expression
     */
    visitLambda(expr: Expr.Lambda): ITypeResultExpr<IType>;
    visitSuffixTrailer(suffixTerm: SuffixTerm.SuffixTrailer, [builder]: [SuffixTypeBuilder]): Diagnostics;
    visitSuffixTermInvalid(_: SuffixTerm.Invalid, [builder]: [SuffixTypeBuilder]): Diagnostics;
    visitSuffixTerm(suffixTerm: SuffixTerm.SuffixTerm, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * Handle the weird edge cases around suffix methods with zero parameters
     * @param trailers suffix term trailers
     * @param builder the suffix type builder
     * @param errors current collection of errors encountered
     */
    private checkSuffixTermTrailers;
    /**
     * Check if this is an implicit call scenario and get add it's return type if necessary
     * @param builder the suffix type builder
     * @param trailer the suffix trailer if it exists
     */
    private checkCall;
    /**
     * Visit a call expression and check for type errors
     * @param call the current call expression
     * @param builder current resolved suffix expression
     */
    visitCall(call: SuffixTerm.Call, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * Resolve variadic call for type errors
     * @param params parameter types
     * @param call current call expression
     */
    private visitVariadicCall;
    /**
     * Check a normal call signature for type errors
     * @param params the parameter types
     * @param call the call expression
     */
    private visitNormalCall;
    /**
     * visit an array index suffix expression.
     * @param suffixTerm the current array index
     * @param builder the current type
     */
    visitHashIndex(suffixTerm: SuffixTerm.HashIndex, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * visit an array bracket suffix expression.
     * @param suffixTerm the current array bracket expression
     * @param builder the suffix type builder
     */
    visitBracketIndex(suffixTerm: SuffixTerm.BracketIndex, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * visit the suffix term for delegates. This will return a new delegate type
     * @param suffixTerm the current delegate node
     * @param builder the suffix type builder
     */
    visitDelegate(suffixTerm: SuffixTerm.Delegate, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * visit the suffix term for literals.
     * @param suffixTerm literal syntax node
     * @param builder the suffix type builder
     */
    visitLiteral(suffixTerm: SuffixTerm.Literal, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * visit the suffix term for identifier.
     * @param suffixTerm identifier syntax node
     * @param builder the suffix type builder
     */
    visitIdentifier(suffixTerm: SuffixTerm.Identifier, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * visit the suffix term for grouping.
     * @param suffixTerm grouping syntax node
     * @param builder the suffix type builder
     */
    visitGrouping(suffixTerm: SuffixTerm.Grouping, [builder]: [SuffixTypeBuilder]): Diagnostics;
    /**
     * Check if the type is valid in this get context
     * @param range token for the source type
     * @param errors accumulated errors
     * @param source source type
     * @param target target type
     * @returns was an error encountered
     */
    private checkGetter;
    /**
     * Is this tracker a basic tracker
     * @param tracker tracker to inspect
     */
    private isBasicTracker;
    /**
     * Check if unary operator is valid for this type
     * @param expr unary expression
     * @param subExpression the sub expression result
     * @param operatorKind the kind of operator
     */
    private checkUnary;
    /**
     * Check if a logical operator is valid for these operands
     * @param expr underlying expression
     * @param leftResult the left operand result
     * @param rightResult the right operand result
     */
    private checkLogical;
    /**
     * Check if the current operator is valid and it's resulting type
     * @param expr the operator expression
     * @param errors the accumulated errors so far
     * @param leftType the left type
     * @param rightType the right type
     * @param operator the operator to consider
     */
    private checkBinary;
    /**
     * Report an error when an operator is not supported by the provided types
     * @param operatorKind operator kind
     * @param expr expression the error occurred in
     * @param leftType the left type
     * @param rightType the right type
     */
    private operatorError;
    /**
     * Check if any of the available binary operators can work for the other operand
     * @param operators the available operators for a given operator type
     * @param otherType the type of the other operand
     * @param errors the current set of errors
     */
    private tryBinaryOperators;
    /**
     * Return the resultant type with any errors
     * @param type current type
     * @param errors accumulated errors
     */
    private resultExpr;
}
export {};
